#include "types.h"
#include <stdlib.h>
#include <stdio.h>

#define parent(i, d) ((i-1)/d)
#define child(i, k, d) (d*i + k)

void heapifyMinDown(HeapPtr *ptr, int index);
void heapifyMinUp(HeapPtr *ptr, int index);

void   NodeSwap(HeapPtr *ptr, int l_index, int r_index);
int    NodesCompare(Node *fst, Node *snd); // 1 - Greater; -1 - Less
Node*  getSuccessor(HeapPtr *ptr, int index);


HeapPtr *HeapInit(int init_n, int dimension)
{
    HeapPtr *ptr  = (HeapPtr *)calloc(1, sizeof(HeapPtr));
    Node   **head = (Node **)  calloc(init_n, sizeof(Node *));

    ptr -> n = init_n;
    ptr -> nodes_n = 0;
    ptr -> d = dimension;
    ptr -> head = head;

    return ptr;
}

int HeapInsert(HeapPtr *ptr, Node *node)
{
    int nodes_n    = ptr -> nodes_n;
    int array_size = ptr -> n;
    int dimension  = ptr -> d;

    if(array_size <= nodes_n)
        return -1;

    Node **head = ptr -> head;

    node -> index = nodes_n;
    head[nodes_n] = node;
    heapifyMinUp(ptr, nodes_n);

    (ptr -> nodes_n)++;

    return 0;
}

Node *getMin(HeapPtr *ptr)
{
    Node *  min; 
    Node ** head;
    int     nodes_n;

    head = ptr -> head;
    nodes_n = ptr -> nodes_n;

    min = head[0];

    NodeSwap(ptr, 0, nodes_n - 1);

    (ptr -> nodes_n)--;

    heapifyMinDown(ptr, 0);
    return min;
}

int relaxSKP(HeapPtr *ptr, int index, int newSKP)
{

    Node **head = ptr -> head;
    
    head[index] -> SKP = newSKP;
    heapifyMinUp(ptr, index);

    return 0;
}

void heapifyMinUp(HeapPtr *ptr, int index)
{
    Node** head = ptr -> head;
    int i = index; 
    int d = ptr -> d;

    while(i >= 0 && NodesCompare( head[i], head[parent(i, d)] ) == -1 )
    {
       NodeSwap(ptr, i, parent(i, d));
       i = parent(i, d);
    }
}

void heapifyMinDown(HeapPtr *ptr, int index)
{
    Node** head = ptr -> head;
    Node*  succ;
    int i = index;
    int n = ptr -> nodes_n;
    int d = ptr -> d;

    //succ = getSuccessor(ptr, i);
    
    while(child(i, 1, d) < n && NodesCompare(succ, head[i]) == -1)
    {
        succ = getSuccessor(ptr, i);
        int succ_index = succ -> index;
        NodeSwap(ptr, i, succ_index);
        i = succ_index;
    }
}

void   NodeSwap(HeapPtr *ptr, int l_index, int r_index)
{
    Node **head;
    Node *tmp;

    head = ptr -> head;

    head[l_index] -> index = r_index;
    head[r_index] -> index = l_index;
    tmp = head[l_index];

    head[l_index] = head[r_index];
    head[r_index] = tmp;

}

int    NodesCompare(Node *fst, Node *snd) // 1 - Greater; -1 - Less
{
    int result = 0;

    if( (fst -> SKP) > (snd -> SKP))
        result = 1;
    else if( (fst -> SKP) < (snd -> SKP) )
        result = -1;

    return result;
}

Node*  getSuccessor(HeapPtr *ptr, int index)
{
    Node** head;
    Node*  succ;
    int    d;
    int    nodes_n;
    int    fst_chld_index;
    int    lst_chld_index;
    int    i;

    head    = ptr -> head;
    d       = ptr -> d;
    nodes_n = ptr -> nodes_n;

    fst_chld_index = child(index, 1, d);
    lst_chld_index = child(index, d, d);

    succ = head[fst_chld_index];

    for(i = fst_chld_index + 1; i <= lst_chld_index && i < nodes_n; i++)
    {
        if(NodesCompare(succ, head[i]) == 1)
            succ = head[i];
    }

    return succ;
}
/*
Node *createEmpty(int index);
void printNode(Node *node);
void printHeap(HeapPtr *ptr);

int main()
{
    int n,d;
    int i;
    HeapPtr *ptr;

    printf("n d\n");
    scanf("%d %d", &n, &d);

    ptr = HeapInit(n, d);

    for(i = 0; i < n; i++)
        if(HeapInsert(ptr, createEmpty(i)) == -1)
            printf("Insert error!\n");

    relaxSKP(ptr, 3, 1);

    printHeap(ptr);
}

void printHeap(HeapPtr *ptr)
{
    while(ptr -> nodes_n > 0)
        printNode(getMin(ptr));
}

void printNode(Node *node)
{
    printf("Node %d \n\t SKP %d\n", node -> graph_index, node -> SKP);
}

Node *createEmpty(int index)
{
    Node *res = calloc(1, sizeof(Node));

    res -> SKP = index;
    res -> graph_index = index;
    res -> edges_num = 0;
    res -> edges = NULL;
    res -> last  = NULL;
    res -> index = 0;

    return res;
}
*/
