#ifndef __DCONFIG__
#define __DCONFIG__
#include <vector> 
#include <boost/heap/binomial_heap.hpp>
#include <cstdint> 
#include <limits.h>
#define INF INT_MAX-250 

typedef struct Road
{
    uint8_t from;
    uint8_t to;
    uint8_t type;
    float cost;

    Road(uint8_t from, uint8_t to, uint8_t type, float cost)
    {
        this -> from = from;
        this -> to   = to;
        this -> type = type;
        this -> cost = cost;
    }
} Road;

typedef struct Node 
{
    std::vector<Road> roads;
    float KMC;
    uint8_t parrent_index;
    uint8_t parrent_road_type;
    
    Node(float KMC)
    {
        this -> KMC = KMC;
    }

    bool operator<(Node const &other) const
    {
        return other.KMC > KMC;
    }
} Node;

struct compare_node
{
    bool operator()(const Node &fst, const Node &snd) const
    {
        return fst.KMC > snd.KMC;
    }
};

typedef std::vector<Node> Graph;
typedef boost::heap::binomial_heap<Node, boost::heap::compare<compare_node>, boost::heap::mutable_<true> > NodesQueue;
typedef NodesQueue::handle_type NodeHandle;

void dijkstraSolve(Graph &net, uint8_t start_index, uint8_t end_index, std::vector<std::pair<int, int> > &fastest_path);

#endif
