#include "DSU.h"
#include <iostream>
//include "types.h"

using namespace std;

typedef std::pair<std::pair<int,int>,int> Edge; //index from + index to + weight = Edge
typedef std::vector< std::vector<Edge> > Graph;
vector<Edge> GenGraph(int nodes_n, int edges_num, int up);

bool UpdateMSTState(vector<Edge> &edges, vector<int> &state, DSU &myDSU)
{
    bool result = false;
    int i;

    for(i = 1; i < edges.size(); ++i)
    {
        Edge current = edges[i];
        int from,to, fromLead, toLead;
        int weight = current.second;
        from = current.first.first;
        to   = current.first.second;

        fromLead = myDSU.SetFind(from);
        toLead   = myDSU.SetFind(to);

        if(fromLead != toLead)
        {
            if(!state[fromLead])
            {
                result = true;
                state[fromLead] = i;
            }
            else
            {
                int LeadWeight = edges[state[fromLead]].second;
                if(weight < LeadWeight) 
                    state[fromLead] = i;
            }

            if(!state[toLead])
            {
                result = true;
                state[fromLead] = i;
            }
            else
            {
                int LeadWeight = edges[state[toLead]].second;
                if(weight < LeadWeight) 
                    state[toLead] = i;
            }
        }   
    }

    return result;
}

vector<Edge> SolveBoruvka(vector<Edge> &edges, int nodes_n)
{
    vector<Edge> theAnswer; // Oo
    vector<int>  state(edges.size());
    DSU myDSU(edges.size());

    edges[0].second = 0;
    for(int i = 0; i < state.size(); i++)
    {
        state[i] = 0;    
        myDSU.SetCreate(i);
    }

    while(UpdateMSTState(edges, state, myDSU))
    {
        for(int i = 1; i <= nodes_n; ++i)
        {
            if(state[i])
            {
                int edge_index = state[i];

                int from = edges[edge_index].first.first;
                int to   = edges[edge_index].first.second;

                int fromLead = myDSU.SetFind(from);
                int toLead   = myDSU.SetFind(to);

                if(fromLead != toLead)
                {
                    theAnswer.push_back(edges[edge_index]);
                    myDSU.SetUnion(from, to);
                }
                state[i] = 0;
            }
        }
    }

    return theAnswer;
}

int main(void)
{
    int nodes_n, edges_n, up;

    scanf("%d %d %d", &nodes_n, &edges_n, &up);

    vector<Edge> edges = GenGraph(nodes_n, edges_n, up);

    for(int i = 0; i < edges.size(); ++i)
        printf("Edge from %d to %d weight %d\n", edges[i].first.first, edges[i].first.second, edges[i].second);


    printf("\n\n--Result--\n\n");

    vector<Edge> answer = SolveBoruvka(edges, nodes_n);

    for(int i = 0; i < answer.size(); ++i)
        printf("Edge from %d to %d weight %d\n", answer[i].first.first, answer[i].first.second, answer[i].second);
    return 0;
}
